#$Id: ModDM.pm,v 1.8 2005/11/23 16:34:58 uhtu Exp $

# Copyright (c) 2003-2005 Hugh Kennedy <kennedyh@engin.umich.edu>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModDM;

# dungeon-master module for omnicron. "roll 3 d5"
# "roll <n> d<m>"   -> roll n m-sided dice
# "roll soc"        -> roll 2 6-sided dice + pirate die
# "roll percentile" -> roll d% via two d10
# "roll roshambo"   -> break a two way tie
#

@ModDM::ISA = qw(OmniMod);

use strict;
my $MAX_COUNT = 10;
my $MAX_SIDES = 100;
my $CARTMAN   = 0.000001;

# the irc colorspec, in all its horrible glory:
# http://www.felmlee.com/bodebot/docs/colors.txt

# primes are pirates, obviously. behold, the hash ARRGH:
my %ARRGH = (
             1 => "\cc2blue\cc",
             2 => "ARRGH",
             3 => "ARRGH",
             4 => "\cc8yellow\cc",
             5 => "ARRGH",
             6 => "\cc3green\cc",
            );

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self  = $class->SUPER::new();
  bless ($self, $class);          # reconsecrate
  return $self;
}

sub help {
  if ( defined wantarray ) { # do we have help?
      return 1;
  }

  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  $irc->notice( $nick, "[roll <n> d<m>]  ->  roll n m-sided dice" );
  $irc->notice( $nick, "[roll soc]  ->  roll 2 6-sided dice + pirate die" );
  $irc->notice( $nick, "[roll percentile] -> roll d% via two d10" );
  $irc->notice( $nick, "[roll roshambo] -> break a two way tie" );
}

sub name() {
  my $this = shift;
  return "dungeonmaster";
}

sub handlePrivateMessage {
  my ($this, $irc, $nick, $message, $event, $conn ) = @_;
  return handleMessage($this, $irc, $nick, $nick, $message);
}

sub handleMessage {
  my ($this, $irc, $channel, $nick, $message ) = @_;
  if ( $message =~ m@^roll (\d+) d(\d+)\s*$@ ) {
    my $count = $1;
    my $sides = $2;
    
    if ( $count < 1 || $sides < 2 ) {
      $irc->privmsg( $channel, "sorry $nick, i'm bound by the laws of physics" );
    } elsif ( $count > $MAX_COUNT ) {
      $irc->privmsg( $channel, "sorry $nick, i didn't bring that many dice" );
    } elsif ( $sides > $MAX_SIDES ) {
      $irc->privmsg( $channel, "sorry $nick, i left all my big dice at home" );
    } else {
      my @results = roll( $count, $sides );
      my $results = join " ", @results;	
      $irc->privmsg( $channel, "i rolled $results" );
    }
  } elsif ( $message =~ m@^roll soc$@ ) {
    my $count = 3;
    my $sides = 6;
    my ( $normal, $red, $bart ) = roll( $count, $sides );
    $irc->privmsg( $channel, "i rolled $normal \cc04$red\cc " . $ARRGH{$bart} );
  } elsif ( $message =~ m@^roll percentile$@ ) {
    # percentile die is a 00-90 die and a 0-9 die 
    my $count = 2;
    my $sides = 10;
    my ($ones, $tens) = roll( $count, $sides );
    $tens--;
    $tens *= 10;
    $ones--; 
    my $tot = $tens + $ones;
    $tens = sprintf("%02d",$tens);
    $irc->privmsg( $channel, "i rolled $tot ($tens $ones)" ); 
  } elsif ( $message =~ m@^roll roshambo$@ ) {
    # rock paper scissors
    my $count = 1;
    my $sides = 3; 
    my $rps = ("", "rock", "paper", "scissors")[ roll( $count, $sides) ]; 
    if ( rand() < $CARTMAN ) {
      $irc->privmsg( $channel, "i'm gonna kick you in the nuts!" );
    } else {
      $irc->privmsg( $channel, "1... 2... 3... $rps!" );
    }
  }
}

sub roll {
  my($count, $sides) = @_;
  my @f;
  $#f = $count-1;
  return map {int rand($sides)+1} @f;
}

1;
