#$Id: ModNickometer.pm,v 1.2 2004/08/26 23:32:23 uhtu Exp $
#
# Copyright (c) 2004 Hugh Kennedy <kennedyh@engin.umich.edu>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
package ModNickometer;


# blatant theft from benbow:
# the 'lame nick-o-meter' reading for m1ght13r is
#                 99.58%, bill

@ModNickometer::ISA = qw(OmniMod);

use strict;
use Math::Trig;

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self  = $class->SUPER::new();
  bless ($self, $class);          # reconsecrate
  return $self;
}

sub help {
  if ( defined wantarray ) { # do we have help?
      return 1;
  }

  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  $irc->notice( $nick, "[nickometer nick]  ->  evaluate nick quality" );
}

sub name() {
  my $this = shift;
  return "nickometer";
}

sub handlePrivateMessage {
  my ($this, $irc, $nick, $message, $event, $conn ) = @_;
  return handleMessage($this, $irc, $nick, $nick, $message);
}

sub handleMessage {
  my ($this, $irc, $channel, $nick, $message ) = @_;
  if ( $message =~ m@^nickometer ([^\s]+)\s*$@ ) {
     my $badnick = $1;
     my $rating = nickometer( $badnick );
     $irc->privmsg( $channel, "the lame-nick-o-meter reading for $badnick is $rating%, $nick" );
  } 
}

# all the rest of this is from 
# http://www.adamspiers.org/computing/nickometer/nickometer.pl
#  (c) 1998 Adam Spiers <nickometer@adamspiers.org>
#
# You may do whatever you want with this code, but give me credit.
#
sub nickometer ($) {
  local $_ = shift;

  my $score = 0;

  # Deal with special cases (precede with \ to prevent de-k3wlt0k)
  my %special_cost = (
		      '69'			=> 500,
		      'dea?th'			=> 500,
		      'dark'			=> 400,
		      'n[i1]ght'		=> 300,
		      'n[i1]te'			=> 500,
		      'fuck'			=> 500,
		      'sh[i1]t'			=> 500,
		      'coo[l1]'			=> 500,
		      'kew[l1]'			=> 500,
		      'lame'			=> 500,
		      'dood'			=> 500,
		      'dude'			=> 500,
		      'rool[sz]'		=> 500,
		      'rule[sz]'		=> 500,
		      '[l1](oo?|u)[sz]er'	=> 500,
		      '[l1]eet'			=> 500,
		      'e[l1]ite'		=> 500,
		      '[l1]ord'			=> 500,
		      'k[i1]ng'			=> 500,
		      'pron'			=> 1000,
		      'warez'			=> 1000,
		      'xx'			=> 100,
		      '\[rkx]0'			=> 1000,
		      '\0[rkx]'			=> 1000,
		     );

  foreach my $special (keys %special_cost) {
    my $special_pattern = $special;
    my $raw = ($special_pattern =~ s/^\\//);
    my $nick = $_;
    unless ($raw) {
      $nick =~ tr/023457+8/ozeasttb/;
    }
    $score = &punish($special_cost{$special}, "matched special case /$special_pattern/", $score)
      if $nick =~ /$special_pattern/i;
  }
  
  # Allow Perl referencing
  s/^\\([A-Za-z])/$1/;
  
  # By his own admission, the nick ^Pudge is slightly lame ...
# s/\^(pudge)/$1/i;

  # C-- ain't so bad :-)
  s/^C--$/C/;
  
  # Punish consecutive non-alphas
  s/([^A-Za-z0-9]{2,})
   /my $consecutive = length($1);
    $score = &punish(&slow_pow(10, $consecutive), 
	    "$consecutive total consecutive non-alphas", $score)
      if $consecutive;
    $1
   /egx;

  # Remove one layer of balanced brackets and punish for rest
  if (s/^([^{}]*)   (\{) (.*) (\}) ([^{}]*)   $/$1$3$5/x ||
      s/^([^\[\]]*) (\[) (.*) (\]) ([^\[\]]*) $/$1$3$5/x ||
      s/^([^()]*)   (\() (.*) (\)) ([^()]*)   $/$1$3$5/x) 
  {
    #print "Removed $2$4 outside parentheses; nick now $_\n" if $verbose;
  }
  my $parentheses = tr/(){}[]/(){}[]/;
  $score = &punish(&slow_pow(10, $parentheses), 
	  "$parentheses extraneous " .
	    ($parentheses == 1 ? 'parenthesis' : 'parentheses'), $score)
    if $parentheses;

  # Punish k3wlt0k
  my @k3wlt0k_weights = (5, 5, 2, 5, 2, 3, 1, 2, 2, 2);
  for my $digit (0 .. 9) {
    my $occurrences = s/$digit/$digit/g || 0;
    $score = &punish($k3wlt0k_weights[$digit] * $occurrences * 30,
	    $occurrences . ' ' .
	      (($occurrences == 1) ? 'occurrence' : 'occurrences') .
	      " of $digit", $score)
      if $occurrences;
  }

  # An alpha caps is not lame in middle or at end, provided the first
  # alpha is caps.
  my $orig_case = $_;
  s/^([^A-Za-z]*[A-Z].*[a-z].*?)[_-]?([A-Z])/$1\l$2/;
  
  # A caps first alpha is sometimes not lame
  s/^([^A-Za-z]*)([A-Z])([a-z])/$1\l$2$3/;
  
  # Punish uppercase to lowercase shifts and vice-versa, modulo 
  # exceptions above
  my $case_shifts = &case_shifts($orig_case);
  $score = &punish(&slow_pow(9, $case_shifts),
	  $case_shifts . ' case ' .
	    (($case_shifts == 1) ? 'shift' : 'shifts'), $score)
    if ($case_shifts > 1 && /[A-Z]/);

  # Punish lame endings (TorgoX, WraithX et al. might kill me for this :-)
  $score = &punish(50, 'last alpha lame', $score) if $orig_case =~ /[XZ][^a-zA-Z]*$/;

  # Punish letter to numeric shifts and vice-versa
  my $number_shifts = &number_shifts($_);
  $score = &punish(&slow_pow(9, $number_shifts), 
	  $number_shifts . ' letter/number ' .
	    (($number_shifts == 1) ? 'shift' : 'shifts'), $score)
    if $number_shifts > 1;

  # Punish extraneous caps
  my $caps = tr/A-Z/A-Z/;
  $score = &punish(&slow_pow(7, $caps), "$caps extraneous caps", $score) if $caps;

  # Now punish anything that's left
  my $remains = $_;
  $remains =~ tr/a-zA-Z0-9//d;
  my $remains_length = length($remains);

  $score = &punish(50 * $remains_length + &slow_pow(9, $remains_length),
	  $remains_length . ' extraneous ' .
	    (($remains_length == 1) ? 'symbol' : 'symbols'), $score)
    if $remains;

  #printf "\nRaw lameness score is %.2f\n", $score if $verbose;

  # Use an appropriate function to map [0, +inf) to [0, 100).
  # Improvements welcome, this one is a bit sucky.
  my $percentage = 100 * 
                     (1 + tanh(($score-400)/400)) * 
                     (1 - 1/(1+$score/5)) / 2;

  my $digits = 2 * (2 - &round_up(log(100 - $percentage) / log(10)));

  return sprintf "%.${digits}f", $percentage;
}

sub case_shifts ($) {
  # This is a neat trick suggested by freeside.  Thanks freeside!

  my $shifts = shift;

  $shifts =~ tr/A-Za-z//cd;
  $shifts =~ tr/A-Z/U/s;
  $shifts =~ tr/a-z/l/s;

  return length($shifts) - 1;
}

sub number_shifts ($) {
  my $shifts = shift;

  $shifts =~ tr/A-Za-z0-9//cd;
  $shifts =~ tr/A-Za-z/l/s;
  $shifts =~ tr/0-9/n/s;

  return length($shifts) - 1;
}

sub slow_pow ($$) {
  my ($x, $y) = @_;

  return $x ** &slow_exponent($y);
}

sub slow_exponent ($) {
  my $x = shift;

  return 1.3 * $x * (1 - atan($x/6) *2/pi);
}

sub round_up ($) {
  my $float = shift;

  return int($float) + ((int($float) == $float) ? 0 : 1);
}

sub punish ($$$) {
  my ($damage, $reason,$score) = @_;

  return $score unless $damage;

  $score += $damage;
  #printf "%.2f lameness points awarded: $reason\n", $damage if $verbose;
  return $score;
}

1;
