# $Id: ModQuote.pm,v 1.20 2006/01/31 16:24:37 uhtu Exp $
# Copyright (c) 2002-2004 Phil Tom <phil@asdf.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModQuote;

@ModQuote::ISA = qw(OmniMod);

use strict;

use URI::Escape;
use HTTP::Cookies;

my $on = 1;
my $ua = new LWP::UserAgent;
$ua->agent('Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.2) Gecko/20010726 Netscape6/6.1');
$ua->timeout(5); # 5s timeout

my $cookie_jar = HTTP::Cookies->new();
$ua->cookie_jar($cookie_jar);

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  $irc->privmsg( $channel, &do_quote( split /\s+/, $message ) );
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  $irc->notice( $nick, &do_quote( split /\s+/, $message ) );
}

sub applicable {
  my $message = shift;
  if ( $message =~ /^quote\s+/ ) {
    return 1;
  }
  return 0;
}

sub do_quote {
  my @words = @_;

  if ( $words[1] =~ /^(djia?)|(dow)|(nasdaq)|(s\&p)$/si || $words[1] =~ /^\^/si ) {
    return get_market_quote(@words);
  }
  return get_stock_quote(@words);
}

sub get_stock_quote {
  my $arg;
  my @words = @_;
  my $retval = '';
  my $delta = '';
  my $pos = 0;
  my $gt = 0;
  my $lt = 0;
	my $curquote;
	my $sign;
  my $sym = $words[1];
  my $qsym = uri_escape($sym);
  my $url = "http://finance.yahoo.com/q?s=$qsym";
  my $request = HTTP::Request->new(GET => $url);
  my $response = $ua->request($request);

  my $html = $response->as_string();

  # get current price
  $html =~ s/\&nbsp;//sgi;

  if ($html =~ m@Last Trade:</td>\s*<td[^>]*><big><b>([\d,\.]+)</b>@si ) {
    $curquote = $1;
  }
  else {
    #print STDERR "html:$html:\n";
    return "$sym?";
  }

  # get delta
  $html =~ m@Change:</td>\s*<td[^>]*>\s*<img[^>]* alt="(\w+)"[^>]*>\s*<b[^>]*>([\d,\s\.%\(\)]+)</b>@si;
  $sign = $1;
  $delta = $2;

  #set delta sign
  if ( $sign =~ /up/i ) {
    $sign = "+";
  } else {
    $sign = "-";
  }
  
  my $high = '';
  my $low = '';
  if ( $html =~ m@Day's Range:</td>\s*<td[^>]*>([\d\.]+)\s*-\s*([\d\.]+)@si ) {
    $low = $1;
    $high = $2;
  } 

  # get vol 
  my $vol = '';
  if ( $html =~ m@Volume:</td>\s*<td[^>]*>([\d,]+)@si ) {
    $vol = $1;
  }

  # get vol avg
  my $volavg = '';
  if ( $html =~ m@Avg Vol <small>\(\d+m\)</small>:</td>\s*<td[^>]*>([\d,]+)@si ) {
    $volavg = $1;
  }

  $retval = $words[1] . " $curquote $sign$delta   |  $high $low  |  $vol (avg: $volavg)";
  $retval =~ s/\s+/ /sgi;

  if ( $words[1] =~ /dvin/si ) {
    my $presplit = $curquote / 25;
    my $dozen = $presplit * 12;
    $retval .= " | pre-rev-split: \$$presplit (\$$dozen a dozen)";
  }
 
  return $retval;
}

sub get_market_quote {
  my @words = @_;
  my $sym = $words[1];
  my $name;
  if    ( $sym =~ /^(djia?)|(dow)$/si ) {
     $name = "DJIA";
     $sym = "^dji";
  }
  elsif ( $sym =~ /^(nas)|(nasdaq)$/si ) {
     $name = "NASDAQ";
     $sym  = "^ixic";
  }
  elsif ( $sym =~ /^(s&p)$/si) {
     $name = "S&P";
     $sym  = "^gspc";
  }

  my $qsym = uri_escape($sym);

  my $url = "http://finance.yahoo.com/q?s=$qsym";
  my $request = HTTP::Request->new(GET => $url);
  my $response = $ua->request($request);
  my $html = $response->as_string();
  $html =~ s/\&nbsp;//sgi;
  my $retval;

  if ( $html =~ m@Index Value:</td>\s*<td[^>]*><big><b>([\d,\.]+)</b>@si ) {
    my $total = $1;

    # get delta
    $html =~ m@Change:</td>\s*<td[^>]*><img[^>]* alt="(\w+)"[^>]*>\s*<b[^>]*>([\d,\s\.%\(\)]+)</b>@si;
    my $sign = $1;
    my $delta = $2;
  
    #set delta sign
    if ( $sign =~ /up/i ) {
       $sign = "+";
     } else {
       $sign = "-";
     }
   
     if ( ! defined $name ) {
       $qsym = quotemeta $sym;
       if ( $html =~ m@<b>([\w\s]+)</b> \([^:]+:$qsym\)@si ) {
           $name = $1;
       } else {
           $name = $sym;
       }
     }
     $retval = "$name $total $sign$delta";
  } else {
     $retval = "$sym ?";
  }

  return $retval;
}

sub name() {
  my $this = shift;
  return "quote";
}

sub help {
    if ( defined wantarray ) { # do we have help?
        return 1;
    }

    my ($thisi,$irc,$nick,$message) = @_;

    $irc->notice( $nick, "quote symbol -> report stock info for symbol" );
}


1;
