#$Id: ModURLPoster.pm,v 1.33 2005/04/09 19:17:42 uhtu Exp $

# Copyright (c) 2003-2005 Hugh Kennedy <kennedyh@engin.umich.edu>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModURLPoster;

# ModURLPoster vets URL posts to a channel.
# if you tell it to unshutup, it will notifiy you when you post
# a URL that is already in its memory model.
#
# if you private chat a url to it via:
#  posturl <channel> <url>
# like
#  posturl #optimus http://mimezine.com/
# the bot will say:
# <omnicron> uhtu suggested http://mimezine.com/ [mimezine - we might be stupid but we're poor]
# to #optimus
#
# a new default mode has been added such that just telling the bot an http url:
# http://mimezine.com/
# will result in a default posting to $DEFAULT_CHANNEL (likely #optimus)
#
# if the URL is not an html page with a <title> that you would like the bot to
# auto-fetch, you can supply it like:
#  posturl #optimus http://mimezine.com/ check this out!
# the bot will say:
# <omnicron> uhtu suggested http://mimezine.com/ 'check this out!'
# to #optimus
#
#  special exception has been made for evil sites, who use hated
#   <div> tags. proposals for meta-data-plugin-modules will be entertained.
#   additional exception for the www.bayarea.com mercury news. also boo on them.
#   more additional exception for reuters.com. asshats.
#   even more for fortune.com. bastards.
#
# tied to the sl cgi. don't update one without the other.

@ModURLPoster::ISA = qw(OmniMod);

use strict;
use HTML::HeadParser;
use HTML::Entities;
use LWP::UserAgent;
use Time::Zone;
use OmniHash;
use POSIX;
use URI;
use CGI;

# XXX: dirty!
use ModChatLog;

# the url we shortlink with
my $SL_URL = "http://mimezine.com/cgi-bin/sl";

# haroostick for where we draw the line
my $BOB_CONSTANT = 1.3; # 30% adjustment in the cutoff
my $CUTOFF_LEN = length($SL_URL) + length("?hhhhh"); # base + the sl value
$CUTOFF_LEN = int($BOB_CONSTANT * $CUTOFF_LEN);

# the longest title we'll print
my $MAX_TITLE = length "this is a really long title that we don't want to print to the channel";

# seconds to timeout network requests
my $TIMEOUT = 5;

# default channel to send to
my $DEFAULT_CHANNEL = '#optimus';

# http://physics.nist.gov/cuu/Units/binary.html
my @SI_PREFIXES = ("", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei");

# use actual instance variables
sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self  = $class->SUPER::new();

  my $ua = LWP::UserAgent->new;
  $ua->agent('Mozilla/5.0 (Doom; U; Doom NT 5.0; en-US; rv:0.9.2) Gecko/20010726 Doomzilla6/6.1');
  $ua->timeout( $TIMEOUT );
  $self->{UA} = $ua;


  # for the reg. those fuckers.
  my $p = HTML::Parser->new(
                            api_version => 3,
                            start_h     => [sub { $self->start(@_) }, "tagname, attr"],
                            end_h       => [sub { $self->end(@_)   }, "tagname"],
                            text_h      => [sub { $self->text(@_)  }, "dtext"],
                           );
  $p->unbroken_text(1);
  $p->report_tags('div','span');

  $self->{PARSER} = $p;

  # seen is a hash keyed by URL, value = "nick, epoch_seconds"

  $self->{SHOWHOSTS} = 1;

  bless ($self, $class);          # reconsecrate
  return $self;
}


sub help {
  if ( defined wantarray ) { # do we have help?
    return 1;
  }

  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  $irc->notice( $nick, "[posturl <channel> <url> [<title>]]  ->  send <url> to <channel>, with <title>" );
  $irc->notice( $nick, "[<url> [<channel> [<title>]]]  ->  send <url> to <channel>, with <title>, defaults to $DEFAULT_CHANNEL" );
  $irc->notice( $nick, "[[un]shutup]  -> toggle telling you about re-posted URLs" );
  $irc->notice( $nick, " see http://mimezine.com/urlposter.html for instructions on how to configure your irc client to enable /url <url> posting");

}

sub name() {
  my $this = shift;
  return "urlposter";
}


sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  my $ua = $this->{UA};
  my $seen = OmniHash::getHash( "seen" );

  if ( $message =~ /^\s*posturl\s+(\S+)\s+(\S+)\s*(.+)?/
      or $message =~ m@^\s*(https?://\S+)\s*(\S+)?\s*(\S+)?@ ) {

    my $chan;
    my $url;
    my $title;
    if ( $message =~ /^\s*posturl\s+(\S+)\s+(\S+)\s*(.+)?/ ) {
      $chan = $1;
      $url  = $2;
      $title = $3;
    }
    elsif ( $message =~ m@^\s*(https?://\S+)\s*(\S+)?\s*(.+)?@ ) {
      $url  = $1;
      $chan = $2;
      $title = $3;

      $chan = $DEFAULT_CHANNEL if ( ! defined $chan );
    }
    $chan = "#" . $chan if ( $chan !~ /^#/ );

    my $size = "";
    my $host = "";
    $title = "'$title'" if defined $title;
    my $chans = OmniHash::getHash( "joinchans" );
    if ( ( $chan !~ /^\s*\x23\w+/ ) or (!defined $$chans{$chan}) ) { # \x23 is '#'
      $irc->notice( $nick, "channel: '$chan' is not a channel" );
    } else {
      if ( $seen->{$url} ) {
        my ($other, $when) = split /,/, $seen->{$url};
	if ( $other eq $nick ) {
	  $other = "you";
	}
        $irc->notice( $nick, "$other already posted that url at ". getLocalTimeFor( $nick, $when ) );
      } else {
        eval {
          my $uri = new URI($url);
          $host = $uri->host;
        };
        if ( $@ ) {
	  $irc->notice( $nick, "$url dosen't appear to be a url" );
          return;
	}

        my $req = HTTP::Request->new(HEAD => $url);
        my $res = $ua->request($req);
        if ( $res->is_success ) {
          if ( $res->content_type =~ /html/i ) {
            if ( ! defined $title ) {
              my $request = HTTP::Request->new(GET => $url);
              my $response = $ua->request($request);
              if ( $response->is_success and $response->content_type =~ /html/i ) {
                my $html = $response->content();

                # XXX: any thoughts on title parser plugins appreciated.
                if ( $url =~ /(?:bayarea.com|reuters.com|siliconvalley.com|fortune.com|philly.com)/ ) {
                  $this->{PARSER}->parse($html);
                  if ( defined $this->{TITLE} ) {
                    $title = $this->{TITLE};
                  }
                  undef $this->{TITLE};

                } else { # everyone else
                  my $p = HTML::HeadParser->new();
                  $p->parse($html);
                  $title = $p->header('Title');
                }
		$title = decode_entities($title); 
		if ( length $title > $MAX_TITLE ) {
                    $title = substr $title, 0, $MAX_TITLE;
                }
                
                $title = "[$title]";
              }
            }
          } else {
            $size = $res->content_length;
            if ( defined $size ) {
              $size = sprintf("(%.1f %sB)", getValuePrefix($size) );
            }
          }
        } else {
          $irc->notice( $nick, "i was unable to access '$url', please check it for accuracy." );
        }
        # we've seen it untranslated
        $seen->{ $url } = "$nick," . time;

        # okay, now we try and shortlink the url
        if ( length $url > $CUTOFF_LEN ) {
          my $eurl = CGI::escape($url);
          $req = HTTP::Request->new(GET => "$SL_URL?url=$eurl");
          $res = $ua->request($req);
          if ( $res->is_success ) {
            my $slurl = $res->headers->header('x-short-url');
            if ( $this->{SHOWHOSTS} ) {
              $url = "$slurl [$host]";
            } else {
              $url = $slurl;
            }
          }
        }

        $title = "" if ! defined $title;
        $size = "" if ! defined $size;

        $irc->privmsg( $chan, "$nick suggests $url $title $size" );
        &ModChatLog::saveMessage($nick, "$nick suggested $url $title $size", $chan );
        # and we've seen it post-translation
        $seen->{ $url } = "$nick," . time;
      }
    }
  } elsif ( $message =~ /shut\s*up/i ) {
    my $shutit = OmniHash::getHash( "shutit" );
    $$shutit{$nick} = ! $$shutit{$nick};
    my $action = $$shutit{$nick} ? "tell you about duplicate urls" : "shut up";
    $irc->notice( $nick, "okay $nick, i'll $action" );
  } elsif ( $message =~ /^showhosts$/i ) {
    $this->{SHOWHOSTS} = ! $this->{SHOWHOSTS};
    my $action = $this->{SHOWHOSTS} ? "show the [host]" : "not show the [host]";
    $irc->notice( $nick, "okay $nick, i'll $action" );
  }
}

sub getValuePrefix($ ) {
   my ($total) = @_;
   my $order = 0;
   my $factor = 1024;
   for ( $order = 0; abs($total) > 1.0; $order++ ) {
       last if ( abs( $total / $factor ) < 1.0 ); 
       $total /= $factor;
   }
   return ($total, $SI_PREFIXES[$order]);
}

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;

  my $seen = $this->{SEEN};

  if ( $message =~ m@\s*(http://\S+)\s*@i ) {
    my $url = $1;
    if ( $seen->{$url} ) {
      my ($other, $when) = split /,/, $seen->{$url};
      my $shutit = OmniHash::getHash( "shutit" );
      return unless $$shutit{$nick};
      $irc->notice( $nick, "fyi: $other already posted that url at ". getLocalTimeFor( $nick, $when ) );
    } else {
      $seen->{ $url } = "$nick," . time;
    }
  }

}


sub getLocalTimeFor {
  my ($nick, $when) = @_;
  # get timezone
  my $numconfig = OmniHash::getHash( "numconfig" );
  my $timezone = $$numconfig{ $nick };
  if ( defined $timezone ) {
    $timezone =~ s/^.*?%//si;
  }
  my $offset = tz_offset( $timezone );
  $offset = tz_offset( 'GMT' )
    if ! defined $offset or $offset =~ /^\s*$/si;

  my $time = strftime( "%H:%M on %D",
                       localtime($when + $offset - tz_local_offset() ) );
  return $time;
}


# html tag parsing for fun and profit.
sub start ($$ ) {
  my ( $self, $tagname, $attr ) = @_;
  if ( $tagname =~ /^(?:div|span)$/i ) {
    if ( $attr->{class} and $attr->{class} =~ /^(?:(?:top)?storyhead(?:line)?|headline)$/i ) {
      $self->{_IN_TITLE} = 1;
    }
  }
}

# html tag parsing for fun and profit.
sub end ($$ ) {
  my ( $self, $tagname ) = @_;
  if ( $tagname =~ /^(?:div|span)$/i ) {
    undef $self->{_IN_TITLE};
  }
}

# html tag parsing for fun and profit.
sub text ($$ ){
  my ( $self, $dtext ) = @_;
  if ( $self->{_IN_TITLE} ) {
    $self->{TITLE} = $dtext;
  }
}

# which oddly enough, is true:
'utterly false';
