# Copyright (c) 2003-2004 Phil Tom <phil@asdf.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

# ModWeather is an OmniMod module.
#
# It checks the weather for a given zipcode.  wunderground.com is the 
# data source.
#
# Syntax:  weather <zipcode>
# $Id: ModWeather.pm,v 1.21 2006/03/06 22:02:15 uhtu Exp $

package ModWeather;

@ModWeather::ISA = qw(OmniMod);

use strict;

use URI::Escape;

my $on = 1;
my $ua = LWP::UserAgent->new;
$ua->agent('Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.2) Gecko/20010726 Netscape6/6.1');
$ua->timeout(5); # 5s timeout

my $FH_THRESHOLD = 95;
my $FC_THRESHOLD = 10;

sub name {
  return "weather";
}

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  $irc->privmsg( $channel, &do_weather( split /\s+/, $message ) );
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  $irc->notice( $nick, &do_weather( split /\s+/, $message ) );
}

sub applicable {
  my $message = shift;
  if ( $message =~ /^weather\s+.{1,40}$/ ) {
    return 1;
  }
  return 0;
}

# weather lookup
sub do_weather {
  my $url;
  my $city = '';
  my $location = '';
  my @words = @_;
  my $temp = '';
  my $deg = '';
  my $cond = '';
  my $feel = 'n/a';
  my $feeldeg = '';

  shift @words;  # remove 'weather' keyword
  if ($words[0]) {
      foreach (@words) {
	  $location .= $_ . ' ';
      }
      chop $location;
  } else {
      return 'weather needs a valid location';
  }

  my $esczip = uri_escape($location);

  my $urlstring = "http://www.wunderground.com/cgi-bin/findweather/getForecast?query=$esczip";
  # print "urlstring: '$urlstring'\n";
  my $request = HTTP::Request->new(GET => $urlstring);
  my $response = $ua->request($request);

  my $text = $response->as_string();
  # if multiple cities match the search term, then we choose the first match
  if ( $text =~ m@<th>Place</th>.*?<a href=\"(.*)\">@si ) {
    $urlstring = "http://www.wunderground.com$1";
    # print "urlstring city: '$urlstring'\n";
    $request = HTTP::Request->new(GET => $urlstring);
    $response = $ua->request($request);
    $text = $response->as_string();
  }
  # catch redirects
  my $count = 0;
  while ( $text =~ /"Refresh" content="0;URL=(.*?)">/si ) {
    $count++;
    last if ( $count > 3 );

    $urlstring = $1;
    # print "urlstring $count: '$urlstring'\n";
    $request = HTTP::Request->new(GET => $urlstring);
    $response = $ua->request($request);
    $text = $response->as_string();
  }

  #print "resp:'$text'\n";
  if ( $text =~ m@Observed at:\s*</td>\s*<td[^>]*>(.*)</td>@i ) {
    #warn "$1";
    $city = $1;
  }
  #if ( $text =~ m@<b>([\d-]+)</b>(?:&nbsp;)&#176;(\w)\s*/@i ) {
  if ( $text =~ m@<nobr><b>([\d\.-]+)</b>&nbsp;&#176;(\w)</nobr>@i ) {
      #warn "$1  $2";
      $temp = $1;
      $deg = $2;
      if ( $temp > $FH_THRESHOLD ) {
        $temp = "fucking hot ($temp $deg)";
    	$deg = " ";
      }
      elsif ( $temp < $FC_THRESHOLD )
      {
        $temp = "cold as fuck ($temp $deg)";
        $deg = " ";
      }
  }
  if ( $text =~ m@&#176;C</nobr>\s*</span>\s*</div>\s*<div id="b" style="font-size: 14px;">([\w\s]+)</div>@si ) {
      #warn "$1";
      $cond = $1;
  }

  # print "'$city' is '$temp' '$deg' and '$cond'. Feels like '$feel' '$feeldeg'\n";
  
  my $returnString;

  if ( $deg eq '' )
  {
    $returnString = "Weather in $location is unknown";
  }
  elsif ( $deg eq " " )
  {
    $returnString = "$city is $temp and $cond.";
  }
  else
  { 
    $returnString = "$city is $temp $deg and $cond.";
  }

  return $returnString;
}

sub help {
    if ( defined wantarray ) { # do we have help?
	return 1;
    }

    my $this = shift;
    my $irc = shift;
    my $nick = shift;
    my $message = shift;

  $irc->notice( $nick, "[weather <location>]  -> checks weather for <location>, where <location> is a zipcode or city" );
}

1;
