# $Id: ModWiGLE.pm,v 1.10 2004/05/16 23:27:26 uhtu Exp $
# Copyright (c) 2002-2004 Robert Hagemann <bobzilla@gothictech.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModWiGLE;

@ModWiGLE::ISA = qw(OmniMod);

use strict;

use URI::Escape;

my $on = 1;
my $ua = LWP::UserAgent->new;
$ua->agent('Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.2) Gecko/20010726 Netscape6/6.1');
$ua->timeout(5); # 5s timeout

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );
  if ( $message =~ /^wigle\s*$/ ) {
    $irc->privmsg( $channel, &get_wigle_stats( split /\s+/, $message ) );
  } elsif ( $message =~ /^wwwd3?\s*$/ ) {
    $irc->privmsg( $channel, &get_wigle_wwwd_stats( split /\s+/, $message ) );
  }
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  if ( $message =~ /^wigle\s*$/ ) {
    $irc->notice( $nick, &get_wigle_stats( split /\s+/, $message ) );
  } elsif ( $message =~ /^wwwd3?\s*$/ ) {
    $irc->notice( $nick, &get_wigle_wwwd_stats( split /\s+/, $message ) );
  }
}

sub applicable {
  my $message = shift;
  if ( $message =~ /^wigle\s*$/ ) {
    return 1;
  } elsif ( $message =~ /^wwwd3?\s*$/ ) {
    return 1;
  }
  return 0;
}

sub get_wigle_stats {
  my @words = @_;

	my $url = 'http://wigle.net/gps/gps/GPSDB/';
  my $request = HTTP::Request->new(GET => $url);
  my $response = $ua->request($request);
  my $html = $response->as_string();
	
	my $retval = "No results";
	if ( $html =~ /mapping of (.*?) wireless networks/si ) {
		my $num = $1;
		$retval = "There are $num wireless networks on WiGLE.net";
	}

	return $retval;
}

sub get_wigle_wwwd_stats {
  my $url = 'http://wigle.net/gps/gps/GPSDB/stats/';
  my $request = HTTP::Request->new(GET => $url);
  my $response = $ua->request($request);
  my $html = $response->as_string();

  my $retval = "No results";
  if ( $html =~ m@New unique networks during <a href="http://www.worldwidewardrive.org/">wwwd3</a>: ([\d,]+)@si ) {
    my $num = $1;
    $retval = "There are $num new wireless networks from wwwd3 on WiGLE.net";
  }

  return $retval;
}

1;
